<?php


/**
 * Menu callback for [files base]/imagene/[gradient type]/%imagene_image
 *
 * @param $im
 *   Image object, as returned by imagene_image_load()
 */
function imagene_image(stdClass $im) {

  _imagene_include();

  if (file_exists($im->file)) {
    $image = imagecreatefrompng($im->file);
  }
  else if (!user_access('see non-existing imagene images')) {
    // an admin should visit this page, before normal visitors do.
    $image = imagecreatefrompng(drupal_get_path('module', 'imagene') . '/empty.png');
  }
  else {
    // create image
    list($w, $h) = $im->image->getDimensions();
    $image = imagecreatetruecolor($w, $h);
    imagealphablending($image, FALSE);
    imagesavealpha($image, TRUE);
    for ($x = 0; $x < $w; ++$x) {
      for ($y = 0; $y < $h; ++$y) {
        @list($r,$g,$b,$a) = $im->image->getPixelRGBA($x, $y);
        if ($im->image->hasAlpha()) {
          // php does only count alpha up to 127, not 255.
          // Thus, we have to divide alpha by 2.
          $color = imagecolorallocatealpha($image, $r, $g, $b, $a/2);
        }
        else {
          $color = imagecolorallocate($image, $r, $g, $b);
        }
        imagesetpixel($image, $x, $y, $color);
      }
    }
    if (user_access('save imagene images')) {
      // save the image
      imagene_parent_mkdir_rec($im->file);
      imagepng($image, $im->file);
    }
  }

  // send the image
  header('Content-type: image/png');
  imagepng($image);
  imagedestroy($image);
  exit();
}


/**
 * Recursive mkdir.
 * Creates missing parent directories.
 * 
 * @param $path
 *   The full path to a file that is to be generated, not to a directory.
 *   For instance, "path/to/file.png" will only do mkdir("path") and
 *   mkdir("path/to"), but not mkdir("path/to/file.png")
 */
function imagene_parent_mkdir_rec($path) {
  $fragments = explode('/', $path);
  array_pop($fragments);
  $path = implode('/', $fragments);
  if (!is_dir($path)) {
    imagene_parent_mkdir_rec($path);
    mkdir($path);
  }
}




